import { ActivatedRouteSnapshot, DetachedRouteHandle, RouteReuseStrategy } from '@angular/router';

export class CustomReuseStrategy implements RouteReuseStrategy {
  private storedHandles = new Map<string, DetachedRouteHandle>();

  shouldDetach(route: ActivatedRouteSnapshot): boolean {
    return route.data.reuse || false;
  }

  store(route: ActivatedRouteSnapshot, handle: DetachedRouteHandle): void {
    if (route.data.reuse) {
      const key = this.getRouteKey(route);
      this.storedHandles.set(key, handle);
    }
  }

  shouldAttach(route: ActivatedRouteSnapshot): boolean {
    const key = this.getRouteKey(route);
    // console.log('shouldAttach:', this.storedHandles.has(key))
    return this.storedHandles.has(key);
  }

  retrieve(route: ActivatedRouteSnapshot): DetachedRouteHandle | null {
    const key = this.getRouteKey(route);
    return this.storedHandles.get(key) || null;
  }

  shouldReuseRoute(future: ActivatedRouteSnapshot, curr: ActivatedRouteSnapshot): boolean {
    // 精确匹配路由复用
    return future.routeConfig === curr.routeConfig &&
           JSON.stringify(future.params) === JSON.stringify(curr.params);
  }

  private getRouteKey(route: ActivatedRouteSnapshot): string {
    let str = route.pathFromRoot
    .filter(v => v.routeConfig)
    .map(v => v.routeConfig!.path + JSON.stringify(v.params))
    .join('/');
    return str.endsWith('/{}') ? str.slice(0, -3) : str;
  }

  public deleteRouteSnapshot(url: string): void {
    const key = url.replace(/\//g, '_');
    if (this.storedHandles.has(key)) {
      this.storedHandles.delete(key);
    }
  }
}
